
# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           pong.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.pong.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.pong")

# standard modules you probably need
import os,sys,random

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

# containers that can be used globally to store stuff
class Img:
    pass
class Snd:
    pass
class Misc:
    pass
# Options used by the game.
# THESE OPTIONS ARE USED, NO CONFIG FILE EXISTS IN CHILDSPLAY_SP
# These will be merged with the ones from the rcdic.
# When there's a problem with the rcdic we use these as defaults. 
defaults_dict = {'__name__':'default',\
                'sound':'yes',\
                'computerAI':'impossible',\
                'gameplay':'single',\
                'batsize':'72',\
                'batspeed':'12',\
                'ballspeed':'5',\
                'goalsize':'300',\
                'winpoints':'11',\
                'useprofile':'none',\
                'left_keyup':'q',\
                'left_keydown':'a',\
                'right_keyup':'p',\
                'right_keydown':'l'}
                
## set ONEPLAYER to 1 for single player pong
## set to 0 for two players, one human and one computer
## this works by moving the left goal and bat behind the wall.
## So everything works the same as in multiplayer but it looks like single player
ONEPLAYER = 1# how may persons, 1 = 1, 0 = 2 or 1 + computer
PCPLAYER = 0# used only in combination with ONEPLAYER = 0

class Winner:
    def __init__(self,side):
        img = self.load_stuff()
        if side == 'left': pos = (200,150)
        else: pos = (500,150)
        Snd.winner.play()
        pygame.display.update(Img.screen.blit(img,pos))
            
    def load_stuff(self):
        return Img.winner
        
class Loser(Winner):
    def __init__(self,side):
        Winner.__init__(self,side)
    def load_stuff(self):
        return Img.loser

class PcPlayer:
    """The computer player.
    This will control the bat by adding pygame events to the pygame event queue.
    """
    def __init__(self, rc_dic):
        """bat must be a Bat class object."""
        self.up,self.down = ord(Misc.bat_l.up),ord(Misc.bat_l.down)
        self.u_up,self.u_down = unicode(Misc.bat_l.up), unicode(Misc.bat_l.down)
        self.batstarty = Misc.bat_l.rect[1]
        self.balldirection = Misc.ball.xoffset
        self.event_dict = {'key':self.up,'unicode':self.u_up}
        self.keystate = KEYUP
        #self.counter = 2# about 0.15 second (clockrate chidsplay 30/sec)
        self.AIlist = (0,)*(int(rc_dic['ballspeed'])-1) + (1,)
        Misc.bat_l.init_speed = 6
        if rc_dic['computerAI'] == 'hard':
            self.AIlist = (0,0, 1)
        elif rc_dic['computerAI'] == 'impossible':
            self.AIlist = (0,0)
        
    def play(self,events):
#        if self.counter > 0: 
#            self.counter -= 1
#            return events
#        else:
#            self.counter = 2
        if self.keystate is KEYDOWN:
            self.keystate = KEYUP
            events.append(pygame.event.Event(self.keystate,self.event_dict))
            return events
        #the ball is moving away from us and we're not back in the middle
        elif Misc.ball.xoffset > 0 and \
                Misc.bat_l.rect[1] == self.batstarty:
            return events  
        elif self.keystate is KEYUP:
            self.keystate = KEYDOWN
        if self.keystate is KEYUP and random.choice(self.AIlist):
            return events        
        bally = Misc.ball.rect[1]
        baty = Misc.bat_l.rect.inflate(0,-4)[1]
        #print bally,baty
        if bally > baty:
            self.event_dict['key'] = self.down
            self.event_dict['unicode'] = self.u_down
        else:
            self.event_dict['key'] = self.up
            self.event_dict['unicode'] = self.u_up
        event = pygame.event.Event(self.keystate,self.event_dict)
        events.append(event)
        return events

class ScoreBoard:
    def __init__(self,rc_dic, end=11,size=48):
        try:
            s = int(rc_dic['winpoints'])
        except (TypeError,KeyError):
            pass
        else: end = s
        self.pos_l,self.pos_r = (340,20),(410,20)
        self.size = size
        self.end = end#at which score the game ends
        self.score = (0,0)
        self.black = pygame.Surface((48,48))
                
    def set_score(self,score,side):
        if side == 'right':
            self.score = (self.score[0],self.score[1]+score)
        else:
            self.score = (self.score[0]+score,self.score[1])
        self.update()            
        
    def update(self):
        rects = []
        sr = self.black.convert()
        sr.blit(utils.char2surf(str(self.score[1]),self.size,GREEN),(0,0))
        sl = self.black.convert()
        sl.blit(utils.char2surf(str(self.score[0]),self.size,GREEN),(0,0))
        rects.append(Img.screen.blit(sl,(self.pos_l)))
        rects.append(Img.screen.blit(sr,(self.pos_r)))
        rects.append(Img.backgr.blit(sl,(self.pos_l)))
        rects.append(Img.backgr.blit(sr,(self.pos_r)))
        pygame.display.update(rects)
        if self.end in self.score:
            # hack to erase the sprite. Because the position of the sprite is changed
            # after it's drawn on the screen. So we must erase it's 'old' position.
            Misc.ball.rect = Misc.ball.oldrect
            Misc.ball.erase_sprite()
            pygame.time.wait(3000)
            if self.score[0] == self.end:
                Winner('left')
                Loser('right')
            else:
                Winner('right')
                Loser('left')
            pygame.time.wait(9500)
            # End of this game
            self.score = (0,0)
            self.update()

class Goal:
    def __init__(self,rc_dic, scoreboard, size=(20,300)):
        try:
            s = int(rc_dic['goalsize'])
        except (TypeError,KeyError):
            pass
        else: size = (20,s)
        self.image = pygame.Surface(size).convert()
        self.score = 0
        #self.image.fill((255,255,255))# use to display goals (debugging)
        self.rect = self.image.get_rect()
        self.scoreboard = scoreboard
    def set_scoreboard(self,side):
        self.side = side#this should be the other side
    def move(self,pos):
        self.rect.move_ip(pos)
    def scored(self):
        Snd.goal.play()
        self.score += 1
        self.scoreboard.set_score(1,self.side)
    def get_score(self):
        return self.score    

class Ball(SPSpriteUtils.SPSprite):
    def __init__(self,rc_dic, speed=4):
        try:
            s = int(rc_dic['ballspeed'])
        except (TypeError,KeyError):
            pass
        else: speed = s
        self.image = pygame.Surface((24,24)).convert()
        pygame.draw.circle(self.image,\
                           GREEN,\
                           (12,12),\
                           12,\
                           0)
        self.image.set_colorkey(self.image.get_at((0, 0)), RLEACCEL)
        SPSpriteUtils.SPSprite.__init__(self,self.image)
        self.rect = self.image.get_rect()
        self.xpos,self.ypos = 390,250
        self.rect.move_ip(self.xpos,self.ypos)
        self.xoffset = random.choice((-speed,speed))
        self.yoffset = random.choice((-speed,speed))
        self.move()
        
    def move(self):
        self.oldrect = self.rect
        if self.rect.centerx > 764:
            Snd.bump.play()
            self.xoffset = -self.xoffset
        if self.rect.centerx < 37:
            Snd.bump.play()
            self.xoffset = abs(self.xoffset)
        if self.rect.centery > 464:
            Snd.bump.play()
            self.yoffset = -self.yoffset
        if self.rect.centery < 36:
            Snd.bump.play()
            self.yoffset = abs(self.yoffset)
        self.rect.centerx += self.xoffset
        self.rect.centery += self.yoffset

    def update(self,*args):
        self.move()
        for obj in (Misc.goal_l,Misc.goal_r):
            if self.rect.colliderect(obj.rect):
                obj.scored()
                return -2
        collide = self.rect.colliderect(Misc.bat_r.rect) or \
                    self.rect.colliderect(Misc.bat_l.rect)
        if collide:
            self.xoffset = -self.xoffset
            Snd.pong.play()

class Bat(SPSpriteUtils.SPSprite):
    def __init__(self,rc_dic, action_keys,size=64,speed=12):
        self.current_key = None#used to fix the "no unicode key in keyup event" pygame bug
        try:
            s = int(rc_dic['batspeed'])
            bs = int(rc_dic['batsize'])
        except (TypeError,KeyError):
            pass
        else: 
            size = bs
            speed = s
        self.init_speed = speed
        self.speed = 0
        self.size = size
        self.top,self.bottom = 18,482
        self.up,self.down = action_keys[0].upper(),action_keys[1].upper()
        self.image = pygame.Surface((8,size))
        self.image.fill(GREEN)
        SPSpriteUtils.SPSprite.__init__(self,self.image)
        self.rect = self.image.get_rect()
            
    def set_position(self,pos):
        self.rect.move_ip(pos[0],pos[1])
        
    def update(self,*args):
        """This keeps moving until a keyup event is raised"""
        #c = None
        for event in Misc.events:
            if event.type == KEYDOWN:
                try:
                    c = event.unicode.upper()
                except ValueError,info:
                    print info
                    return
                else:
                    if c == self.up:
                        self.speed = self.init_speed
                        self.speed = -self.speed
                        self.current_key = c
                    elif c == self.down:
                        self.speed = self.init_speed
                        self.speed = abs(self.speed)
                        self.current_key = c
                break
            elif event.type == KEYUP and self.current_key:
                self.speed = 0
                self.current_key = None
                break
        newrect = self.rect.move(0,self.speed)
        if newrect.top > self.top and newrect.bottom < self.bottom:
            self.rect = newrect

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.pong.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        Img.screen = self.screen
        Img.backgr = self.backgr
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','PongData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'c'))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        self.rc_dic = defaults_dict
        self.SPG.tellcore_disable_menubuttons()
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Pong")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "pong"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        " ",
        _("The classic pong game where you must hit a ball with your bat."),
        _("There are three modes to choose from:"),\
        _("Single play - Hit the ball against the wall."),\
        _("Multi player against the computer - Try to defeat the computer."),\
        _("Multi player - Play against another player."),\
        _("In the multiplayer modes, the one who has 11 points wins."),\
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Fun")
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        self.pre_level_flag = True
        Img.screen.fill((0,0,0))
        pygame.display.update()
        # Display the three choices of game play.
        # This will put the menu items in the actives group which then will be updated
        # in the game loop. When the users picks one the result will be handled by
        # the restart method.
        self.choice = None # will be set in loop
        # FIXME: disabled the pc pclayer as it's not working right.
        # The bats are not moving smoothly whit the pcplayer.
        # The same problem exists when playing with two players but it's less annoying 
        positions = [(130,70),(130,220)]#,(130,370)]
        images = ('single.jpg','multi_person.jpg')#,'multi_pc.jpg')
        head = _("Choose the game to play:")
        s = utils.char2surf(head,32,GREEN, bold=True)
        pygame.display.update(Img.screen.blit(s,(80,4)))
        for pos,img,data in map(None,positions,images,('1','2')):
            img = os.path.join(self.my_datadir,img)
            obj = SPSpriteUtils.SPButton(utils.load_image(img),pos,data)
            self.actives.add(obj)
            obj.display_sprite() 
        return True
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        return True
    
    def start(self):
        """Mandatory method."""
        self.SPG.tellcore_set_dice_minimal_level(6)
        fs = 36
        self.scoreboard = ScoreBoard(self.rc_dic, size=fs)
        Img.winner = utils.load_image(os.path.join(self.my_datadir,'winner.jpg'),1)
        Img.loser = utils.load_image(os.path.join(self.my_datadir,'loser.jpg'),1)
        self.skipstart = None# used when the user sets a predefined game play in the config file
        self.skipsplash = None
        if self.rc_dic['sound'].lower() == 'no':
            Snd.pong = utils.load_sound(os.path.join(self.my_datadir,''))
            Snd.winner = utils.load_sound(os.path.join(self.my_datadir,''))
            Snd.goal = utils.load_sound(os.path.join(self.my_datadir,''))
            Snd.bump = utils.load_sound(os.path.join(self.my_datadir,''))
        else:
            Snd.pong = utils.load_sound(os.path.join(self.my_datadir,'pick.wav'))
            Snd.winner = utils.load_music(os.path.join(self.my_datadir,'winner.ogg'))
            Snd.goal = utils.load_sound(os.path.join(self.my_datadir,'goal.wav'))
            Snd.bump = utils.load_sound(os.path.join(self.my_datadir,'bump.wav'))
        #set kind of game play
        # we only check for multi and multipc, anything else is considerd single play
        # which is the default
        if self.rc_dic['gameplay'] == 'multi':
            self.restart([[None,'2']])
            self.skipstart = 1
        elif self.rc_dic['gameplay'] == 'multipc':
            self.restart([[None,'3']])
            self.skipstart = 1
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        self.SPG.tellcore_enable_dice(False)
        if not self.skipsplash:
            self._splash_controls()
        self.next_exercise()  
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass
        self.SPG.tellcore_enable_menubuttons()
    
    def next_exercise(self):
        self.logger.debug("ONEPLAYER: %s PCPLAYER: %s" % (ONEPLAYER,PCPLAYER))

        self.PCplayer = None
        self.pcplayer_wait = 0
        self._draw_field()
        self.scoreboard.update()
        Misc.goal_r = Goal(self.rc_dic, self.scoreboard)
        Misc.goal_r.set_scoreboard('left')
        Misc.goal_r.move((774,100))
        Misc.goal_l = Goal(self.rc_dic, self.scoreboard)
        Misc.goal_l.set_scoreboard('right')
        if ONEPLAYER:
            Misc.goal_l.move((-20,100))# move behind the wall
        else:
            Misc.goal_l.move((6,100))
        Img.screen.blit(Misc.goal_r.image,Misc.goal_r.rect)
        Img.screen.blit(Misc.goal_l.image,Misc.goal_l.rect)
                
        pygame.display.update()
        
        lspeed,rspeed = 8,8
        try:
            u = unicode(self.rc_dic['right_keyup'])
            d = unicode(self.rc_dic['right_keydown'])
        except (TypeError,KeyError):
            u,d = u'p',u'l'
        Misc.bat_r = Bat(self.rc_dic, action_keys=(u,d),speed=rspeed)# to keep a reference to the bat, see class Ball
        Misc.bat_r.set_position((768,220))
        try:
            u = unicode(self.rc_dic['left_keyup'])
            d = unicode(self.rc_dic['left_keydown'])
        except (TypeError,KeyError):
            u,d = u'q',u'a'
        Misc.bat_l = Bat(self.rc_dic, action_keys=(u,d),speed=lspeed)
        
        if ONEPLAYER:
            Misc.bat_l.set_position((-20,220))
        else:
            Misc.bat_l.set_position((24,220))
        Misc.ball = Ball(self.rc_dic)# now the computer player can check the ball
        self.actives.add(Misc.ball)
        if PCPLAYER and not ONEPLAYER:#we play against the pc
            self.PCplayer = PcPlayer(self.rc_dic)
        
        self.actives.add([Misc.bat_r,Misc.bat_l])
        r = self.actives.draw(Img.screen)
        pygame.display.update(r)

    def _draw_field(self):
        Img.screen.fill((0,0,0))
        Img.backgr.fill((0,0,0))
        
        box = pygame.Rect(0,0,800,500).inflate(-32,-32)
        #box.inflate(-32,-32)
        pygame.draw.rect(Img.screen,\
                         GREEN,\
                         box,\
                         4)
        pygame.draw.line(Img.screen,\
                          GREEN,\
                          (398,16),\
                          (398,484),\
                          4)
        pygame.draw.line(Img.backgr,\
                          GREEN,\
                          (398,16),\
                          (398,484),\
                          4)
        txt = _("Hit the Escape key to stop.")
        s = utils.char2surf(txt, 18, GREEN, bold=True)
        Img.screen.blit(s, (50, 550))
                          
    def _splash_controls(self):
        Img.screen.fill((0,0,0))
        txt = _("Use these keys on your keyboard to control the bat.")
        txtlist = utils.txtfmt([txt],36)
        s1,spam = utils.text2surf(txtlist[0],18,GREEN, bold=True)
        s2,spam = utils.text2surf(txtlist[1],18,GREEN, bold=True)
        Img.screen.blit(s1,(100,200))
        Img.screen.blit(s2,(100,240))
        pygame.display.update()
        rects = []
        fsize = 24
        
        surf = pygame.Surface((60,300))
        surf.blit(utils.load_image(os.path.join(self.my_datadir,'arrow_up.png')),(4,0))
        #surf.blit(utils.load_image(os.path.join(self.my_datadir,'bat.png')),(26,120))
        surf.blit(utils.load_image(os.path.join(self.my_datadir,'arrow_down.png')),(4,240))
        surf_r = surf.convert()# copy for the right side
        
        if ONEPLAYER == 0 and PCPLAYER == 0:
            surf.blit(utils.char2surf(self.rc_dic['left_keyup'].upper(),fsize,GREEN, bold=True),(16,70))
            surf.blit(utils.char2surf(self.rc_dic['left_keydown'].upper(),fsize,GREEN, bold=True),(16,190))
        surf_r.blit(utils.char2surf(self.rc_dic['right_keyup'].upper(),fsize,GREEN, bold=True),(16,70))
        surf_r.blit(utils.char2surf(self.rc_dic['right_keydown'].upper(),fsize,GREEN, bold=True),(16,190))
        rects.append(Img.screen.blit(surf,(40,100)))
        rects.append(Img.screen.blit(surf_r,(700,100)))
        pygame.display.update(rects)
        pygame.time.wait(4000)
        self.skipssplash = 1

    def restart(self):
        """Used to restart the game when there's a goal.
        This is also used to set the global vars when ending the first level(menu choices)"""
        global ONEPLAYER,PCPLAYER
        self.logger.debug("restart called" )
        
        val = self.menuchoice
        if val == -2:#scored
            self.actives.empty()
            self.next_exercise() 
        # Handle the vals from the pre_level
        else:
            if val == '2':# 1 is the default so we don't test it
                ONEPLAYER = 0
            elif val == '3':
                ONEPLAYER = 0
                PCPLAYER = 1
            self.actives.empty()    
            #self.SPG.tellcore_level_end()
    
    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        item = None
        if PCPLAYER:
            # Disabled the pcplayer, see def pre_level
            self.pcplayer_wait += 1
            if self.pcplayer_wait > 10:
                self.pcplayer_wait = 0
                events = self.PCplayer.play(events)# let the computer make his move. 
                #It will add an event to pygame events. Just like a real player :-)
            
        Misc.events = []#This is a hack to be able to use keydown AND keyup with
                        #CPSprite stuff
        for event in events:
            
            if event.type == KEYDOWN or \
                    event.type == KEYUP or\
                    event.type == MOUSEBUTTONDOWN:
                Misc.events.append(event)
                item = event
        result = self.actives.refresh(item)# This will return -2 when there's a score
                    # The menu objects from the pre_level will return strings ('1','2' or '3')
        if result:
            self.menuchoice = result[0][1]
            if self.pre_level_flag and result[0][1] != -2:
                self.restart()
                self.pre_level_flag = False
                for obj in self.actives.get_sprites():
                    obj.erase_sprite()
                    self.actives.empty()
                self.SPG.tellcore_pre_level_end()
            else:
                self.restart()
        
        return 
        



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPContentTables
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# This module parses the content xml files and turns them into sqlite tables.
# The xml files must be named like this: content_<name of the quiz>.xml
# For example the content file for the picture quiz, content_picture.xml

import os
import logging

import glob
import zlib
from SPConstants import *

if NoGtk:
    try:
        from pysqlite2 import dbapi2 as sqlite3
    except ImportError:
        import sqlite3
else:
    import sqlite3

try:
    from xml.etree.ElementTree import ElementTree
except ImportError:
    # try the python2.4 way
    from elementtree.ElementTree import ElementTree

class ParserError(Exception):
    pass


def check_tables(dbase):
    """Called by the core when it starts.
    It checks if all the quiz content tables are in sync with the content xml files.
    It checks for differences in the xml files by checking their adler32 checksums
    against the sum in the adler32 sql table.
    When the sums differ the xml is parsed and a new sql table is created.
    When there's a xml file with no table it will be parsed and stored into a sql table.
    """
    # first check the quizcontent dbase and add any missing table.
    logger = logging.getLogger("childsplay.SPContentTables.check_tables")
    logger.debug("Sync quiz content dbase: %s" % dbase)
    
    contentdir = os.path.join(ACTIVITYDATADIR,'CPData','Quizcontent', '*', 'content_*.xml')
    
    con = sqlite3.connect(dbase)
    cursor = con.cursor()
    cursor.execute(''' SELECT name FROM sqlite_master
                            WHERE type='table' 
                            ORDER BY name;
                            ''')
    tl = [tn[0] for tn in cursor.fetchall()]
    contentfiles = glob.glob(contentdir)
    
    path_types = {}
    for f in contentfiles:
        lang = f.rsplit('/',2)[-2]
        tp = os.path.basename(f).split('_')[1][:-4]
        path_types[f] = tp + '_%s' % lang
    
    for t in path_types.values():
        if t not in tl:
            logger.debug("Creating table '%s'" % t)
            # group is named _group as group is a reserved word in sqlite3
            cursor.execute(''' CREATE TABLE %s
                    ( id INTEGER PRIMARY KEY, content_id TEXT, content_type TEXT,
                    difficulty TEXT, question_text TEXT,
                    question_audio TEXT, answer TEXT, answer_audio TEXT,
                    wrong_answer1 TEXT, wrong_answer1_audio TEXT,
                    wrong_answer2 TEXT, wrong_answer2_audio TEXT,
                    wrong_answer3 TEXT, wrong_answer3_audio TEXT,
                    data TEXT, year TEXT, _group TEXT) 
                    ''' % t)
            con.commit()
            for p, pt in path_types.items():
                if pt == t:
                    break
            if parse_xml(p, cursor, t):
                con.commit()
            
    if 'adler32' not in tl:
        logger.debug("Creating table '%s'" % 'adler32')
        cursor.execute(''' CREATE TABLE %s
                    ( id INTEGER PRIMARY KEY, path TEXT, sum INTEGER, type TEXT) 
                    ''' % 'adler32')
    con.commit()
    
    # Now we know we have all the tables we gonna check the checksums
    cursor.execute(''' SELECT * FROM adler32 ''')
    rowlist = cursor.fetchall()
    # check if we have rows
    if not rowlist:
        logger.debug("no rows found in adler32")
        for p in contentfiles:
            logger.debug("creating row for %s" % p)
            data = open(p, 'r').read()
            cs = zlib.adler32(data)
            cursor.execute(''' INSERT INTO adler32 VALUES (NULL,?,?,?)''', (p, cs, path_types[p]))
            logger.debug("created row for %s with adler %s" % (p, cs))
            # As we don't had adler rows we assume we don't have correct content tables as well.
            # remove content table
            logger.debug("replacing table %s" % path_types[p])
            cursor.execute('''DELETE FROM %s''' % path_types[p])
            con.commit()
            # parse content xml and stote it into the new content table
            if parse_xml(p, cursor, path_types[p]):
                con.commit()
        # just to make sure
        close_dbase(con)
        return
    else: 
        # if we have rows check that we have rows for each content file.
        # The actual files present on disk are leading for this.
        tp = [t[1] for t in rowlist]
        for p in contentfiles:
            if p not in tp:
                data = open(p, 'r').read()
                cs = zlib.adler32(data)
                cursor.execute(''' INSERT INTO adler32 VALUES (NULL,?,?,?)''', (p, cs, path_types[p]))
                logger.debug("created row for %s with adler %s" % (p, cs))
    # renew rowlist
    cursor.execute(''' SELECT * FROM adler32 ''')
    rowlist = cursor.fetchall()
    for row in rowlist:
        p = row[1]
        try:
            data = open(p, 'r').read()
        except IOError:
            logger.warning("Quizcontent file not found: %s" % p)
            logger.warning("replacing table %s" % p)
            cursor.execute(''' DELETE FROM adler32 WHERE id=? ''', (row[0], ))
            lang = p.rsplit('/',2)[-2]
            tp = os.path.basename(p).split('_')[1][:-4]
            ptype = tp + '_%s' % lang
            cursor.execute('''DELETE FROM %s''' % ptype)
            con.commit()
            continue
        cs = zlib.adler32(data)
        logger.debug("checking data from %s with adler %s" % (p, cs))
        #print type(cs), cs, type(row[2]), row[2]
        if cs != row[2]:
            logger.debug("checksum differ for %s" % p)
            # first remove the row from adler32 table
            cursor.execute(''' DELETE FROM adler32 WHERE id=? ''', (row[0], ))
            # add a new one with correct sum
            logger.debug("Added new row for %s" % row[1])
            cursor.execute(''' INSERT INTO adler32 VALUES (NULL,?,?,?)''', (p, cs, path_types[p]))
            con.commit()
            # remove content table
            logger.debug("replacing table %s" % path_types[p])
            cursor.execute('''DELETE FROM %s''' % path_types[p])
            con.commit()
            # and parse content xml and stote it into the new content table
            if parse_xml(p, cursor, path_types[p]):
                con.commit()
    # just to make sure
    close_dbase(con)
    
def close_dbase(con):
    logger = logging.getLogger("childsplay.SPContentTables.close_dbase")
    logger.debug("Commiting transactions and closing dbase.")
    con.commit()    
    con.close()    
    
def parse_xml(xml, cursor, tname):
    """Parses the whole xml tree into a hash with lists. Each list contains hashes
    with the elelements from a 'question' element.
    When year is set only the entries that have a year element and whos value
    is between 'year' and 'year'+9 is put into the hash.
    This is used by the history activity."""  
    logger = logging.getLogger("childsplay.SPContentTables.parse_xml")
    logger.debug("Starting to parse: %s" % xml)
    
    #     xml file:
    #    <?xml version="1.0" encoding="utf-8"?>
    #    <questions>
    #         <question id="2" type="text">
    #                <difficulty>1</difficulty>
    #                <question_text audio="">Wat is een Vlaamse reus?</question_text>
    #                <answer audio="">Een konijnensoort</answer>
    #                <data/><misc></misc>
    #                <wrong_answer1 audio="">Een trekpaard</wrong_answer1>
    #                <wrong_answer2 audio="">De bijnaam voor een opstandeling</wrong_
    #                                           answer2>
    #                <wrong_answer3 audio="">Een grote Belg</wrong_answer3>
    #        </question>
    #       ..... 
    tree = ElementTree()
    tree.parse(xml)
    xml = {}
    # here we start the parsing, the check if we have all the content is done
    # in get_exercise.
    # we don't do it here because of speed considerations.
    index = 0
    questions = tree.findall('question')
    logger.debug("found %s questions in total" % len(questions))

    for q in questions:
        hash = {}
        try:
            hash['content_id'] = q.get('id')
            hash['content_type'] = q.get('type')
            
            e = q.find('difficulty')
            hash['difficulty'] = e.text
            
            e = q.find('data')
            hash['data'] = e.text
            
            e = q.find('group')
            hash['_group'] = e.text
            
            e = q.find('question_text')
            hash['question_text'] = e.text
            hash['question_audio'] = e.get('audio')
            
            e = q.find('answer')
            hash['answer'] = e.text
            hash['answer_audio'] = e.get('audio')
            
            e = q.find('wrong_answer1')
            hash['wrong_answer1'] = e.text
            hash['wrong_answer1_audio']  = e.get('audio')
            e = q.find('wrong_answer2')
            hash['wrong_answer2'] = e.text
            hash['wrong_answer2_audio'] = e.get('audio')
            e = q.find('wrong_answer3')
            hash['wrong_answer3'] = e.text
            hash['wrong_answer3_audio'] = e.get('audio')
            try:
                # year is optional
                e = q.find('year')
                hash['year'] = e.text
            except:
                hash['year'] = ''           
        except AttributeError, info:
            logger.error("The content.xml is badly formed, missing element(s):%s,%s" % (info, e))
            logger.error("question '%s' will be removed from the collection" % q.get('id'))
        else:
            index += 1
            cursor.execute(''' INSERT INTO %s VALUES (\
                            NULL, :content_id, :content_type, :difficulty,\
                            :question_text, :question_audio, :answer,\
                            :answer_audio, :wrong_answer1,\
                            :wrong_answer1_audio, :wrong_answer2,\
                            :wrong_answer2_audio, :wrong_answer3,\
                            :wrong_answer3_audio, :data, :year, :_group\
                            ) ''' % tname , hash)
    logger.debug("Done parsing %s questions" % index)
    return True
    
if __name__ == '__main__':
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    
    lang = 'nl'
    dbase = os.path.join(HOMEDIR, 'quizcontent.db')
    contentdir = os.path.join(ACTIVITYDATADIR,'CPData','Quizcontent', lang, 'content_*.xml')
    check_tables(dbase, contentdir, lang)
      




import sys
sys.path.insert(0, '..')

from base import *
from buttons import *
from dialogs import *
from funcs import *
from text import *
from gtk_widgets import *

import pygame
from pygame.constants import *
import logging 

from SPSpriteUtils import SPSprite, SPGroup
import utils
from SPConstants import *
import SPLogging
 
if __name__ == '__main__':
    
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    
    SPLogging.set_level('debug')
    SPLogging.start()
        
    pygame.init()
    
    from SPSpriteUtils import SPInit
    
    def cbf(sprite, event, data=''):
        print 'cb called with sprite %s, event %s and data %s' % (sprite, event, data)
        print 'sprite name: %s' % sprite.get_name()
        print 'data is %s' % data
    
    def cbf_but(sprite, event, data=''):
        back = pygame.Surface((300,300))
        back.fill(GREEN)
        but = Button('Dialog Button', (10, 10), padding=6)
        but.connect_callback(cbf, MOUSEBUTTONUP, 'Dialog Button')
        objects =[but]
        
        dlg = DialogWindow(back,objects)
        dlg.run()
    
    scr = pygame.display.set_mode((800, 600))
    scr.fill(GREY90)
    pygame.display.flip()
    back = scr.convert()
    actives = SPInit(scr, back)
    
    Init('braintrainer')
    
#    but = Button('Button', (10, 80), padding=6)
#    but.connect_callback(cbf_but, MOUSEBUTTONUP, 'Button')
#    actives.add(but)
#    lbl = Label('Hit escape key to continue', (10, 130))
#    r = pygame.Rect(0, 0, 500, 240)
#    txt = "This program is free software; you can redistribute it and/or modify it under the terms of version 3 of the GNU General Public License as published by the Free Software Foundation.  A copy of this license should be included in the file GPL-3."
#    
#    tv = TextView(txt, (290, 180), r, autofit=True, border=True)
#    tv.display_sprite()
#    
#    va = VolumeAdjust((200, 20))
#    va.display()
#    
#    pnb = PrevNextButton((150, 180), cbf)
#        
#    actives.add([but, lbl, va.get_actives(), pnb.get_actives()])
#    for s in actives:
#        s.display_sprite()
    
    
#    ec = ExeCounter((600, 10), 20)
#    ec.display_sprite()
    
#    box = TextEntryBox((10, 100), height=10, length=500)
#    box.display_sprite()
#    actives.add(box.get_actives())
#
#    TI = TextEntry((10, 10), message="BT2.1")
#    TI.display_sprite()
#    actives.add(TI)

#    objects = []
#    for i in range(30):
#        t = "Label %s" % i
#        lbl = Label(t, (0, 0))
#        lbl.connect_callback(cbf, MOUSEBUTTONUP, t)
#        actives.add(lbl)
#        objects.append(lbl)
#    
#    sw = ScrollWindow((100, 100), (400, 400), objects, border=2)
#    actives.add(sw)
#    actives.add(sw.get_actives())

#    rb = RadioButton(None, "test radiobutton", (100, 100), name='rb')
#    rb.display_sprite()
#    actives.add(rb)
#    
#    rb0 = RadioButton(rb, "test radiobutton", (100, 130), name='rb0')
#    rb0.display_sprite()
#    actives.add(rb0)
    runloop = 1 
    
    pb = ProgressBar((100,200), (300,30), steps=10)
    pb.display_sprite()
    count = 0
    while runloop:
        if count > 5:
            count = 0
            steps = pb.update()
            print steps
            if steps == False:
                runloop = False
        else:
            count += 1
        pygame.time.wait(100)
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type is KEYDOWN:
                if event.key == K_ESCAPE:
                    runloop = 0
#                elif event.key == K_F1:
#                    but.enable(False)
#                    print "button disabled"
#                elif event.key == K_F2:
#                    but.enable(True)
#                    print "button enabled"
#                elif event.key == K_F3:
#                    pnb.toggle()
#                    print "prevnext toggled"
            result = actives.update(event)
            if result and result[0][1] == -1:
                break
                
#    print TI.get_text()
#    print box.get_text()
    raw_input("hit any key to stop")
    


# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           test_act.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.test_act.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.test_act")

# standard modules you probably need
import os
import sys
print sys.path
# Now python finds the pygame 1.9 package first.
# It's ignored when there's no such path
import pygame
module_logger.warning("Using pygame version %s" % pygame.version.ver)
import pygame.camera
from pygame.constants import *
from pygame.locals import *

import mpylayer
import time
from datetime import datetime
import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets


class Capture(object):
    def __init__(self, screen, SPGoodies, size=(512, 384)):
        self.screen = screen
        self.SPG = SPGoodies
        self.size = size
        self.cam = '/dev/video0'
        # We set a flag to check if we have a initialised camera.
        # If the caller accesses it pygame segfaults.
        self.WeHaveCamera = False
        self.blit_pos = (0, 0)

    def __del__(self):
        # This is called when the object is garbage collected by the
        # Python VM. This way we make sure the camera is stopped.
        # We have a lot of problems with video in pygame so I assume
        # that manually stopping the camera is a good thing.
        # But you should never depend on __del__ as its not known when
        # it's actual garbage collected. It's only to make sure it is called
        # sometime. The caller class should call 'stop' when it's finished 
        # with this class.
        self.WeHaveCamera = False
        try:
            self.stop()
        except:
            pass
    
    def start(self):
        pygame.camera.init()
        self.WeHaveCamera = True
        self.camera = pygame.camera.Camera(self.cam, self.size)
        self.snapshot = pygame.surface.Surface(self.size, 0, self.screen)
        self.camera.start() 

    def get_and_flip(self):
        if not self.WeHaveCamera:
            return
        self.snapshot = self.camera.get_image(self.snapshot)
        r = self.snapshot.get_rect()
        self.blit_pos = ((800-r.w)/2 , 100 + (500-r.h)/2)
        self.screen.blit(self.snapshot, self.blit_pos)
        pygame.display.flip()

    def save_image(self,  base_dir = '/tmp'):
        capture_path = os.path.join(base_dir,'braintrainer','capture')
        if not os.path.isdir(capture_path):
            os.mkdir(capture_path)
        filename = os.path.join(capture_path, str(self.SPG.get_current_user_id()) + ".png")
        print "Filename: ", filename
        pygame.image.save(self.snapshot, filename)
        #self.screen.blit(self.snapshot, self.blit_pos)
        #pygame.time.wait(1500)
    
    def stop(self):
        self.WeHaveCamera = False
        self.camera.stop()
        
    def running(self):
        return self.WeHaveCamera        

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.test_act.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies        
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Test_actData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'test_act.rc'))
        self.movie = os.path.join(self.my_datadir,'test1.mpg')
        self.yTop = 100
        self.logger.debug("rchash: " + str(self.rchash))
        #movie stuff
        self.moviePlaying=0
        self.moviePaused=0

        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
                
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Test_act")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "test_act"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        " ",
        _("Number of levels : 5"),
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
    
    def start(self):
        """Mandatory method."""
        self.cap = Capture(self.screen, self.SPG,  size=(320, 240))
        
        b = os.path.join(self.CPdatadir, '200px_80px_blue.png')
        b_ro = os.path.join(self.CPdatadir, '200px_80px_black.png')
        self.shootbut = SPWidgets.TransImgButton(b, b_ro, (300, 500), fcol=WHITE, \
                                             text=_("Shoot"))
        self.shootbut.connect_callback(self._cbf_shoot_but, MOUSEBUTTONUP, None)
        
        mp = mpylayer.MPlayerControl()
        test = os.path.join(ACTIVITYDATADIR, 'CPData', 'VideoData','video', 'test1')
        mp.loadfile(os.path.join(test,'aquaa.mpg'))

    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        self.cap.start()
        self.shootbut.display_sprite()
        self.actives.add(self.shootbut)
        return True
    
    def _cbf_shoot_but(self, widget, event, data):
        # we don't use the args, but it serve as an example.
        self.cap.save_image(HOMEDIR)
    
    def start_help_movie(self):
        pygame.mixer.quit()
        self.movie = os.path.join(self.my_datadir,'test1.mpg')
        self.logger.debug("starting playback of %s" % self.movie)
        self.myMovie = pygame.movie.Movie(self.movie)
        self.myMovie.set_volume(1.0)
        sz = self.myMovie.get_size()
        # set size 300x150 but this is returnt by movie.get_size (624, 352) ???
        #print ">>>>>>>",  'set size', pygame.Rect(250,self.yTop+50,350,self.yTop+200)
        self.myMovie.set_display(self.screen,(250,self.yTop+50,350,self.yTop+200))
        #print ">>>>>>>>", 'actual movie size', self.myMovie.get_size()
        self.myMovie.play()
        
    def stop_help_movie(self):
        self.myMovie.stop()
        pygame.time.wait(1000)
        del self.myMovie
                    
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        pass
        
    def get_dailytraining_result(self):
        """Mandatory method"""
        # TODO: make real result dict
        return {'score':5}
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass
        try:
            self.stop_help_movie()
        except:
            pass
        try:
            self.cap.stop()
        except:
            pass

#    def startMovie(self):
#        self.moviePlaying=1
#        self.moviePaused=0
#        self.logger.debug("starting playback of %s" % self.movie)
#        movie = pygame.movie.Movie(self.movie)
#        movie.set_volume(1.0)
#        sz=movie.get_size()
#        movie.set_display(self.screen,(0,0+self.yTop,800,500))
#        movie.play()
#        return movie
#
#    def pauseMovie(self):
#        self.logger.debug("user paused movie")
#        self.myMovie.set_display(self.screen,(0,0+self.yTop,800,500))
#        self.myMovie.pause()
#        self.moviePlaying=1
#        self.moviePaused=1
#
#    def unPauseMovie(self):
#        self.logger.debug("user continued movie")
#        self.myMovie.set_display(self.screen,(0,0+self.yTop,800,500))
#        self.myMovie.pause()
#        self.moviePlaying=1
#        self.moviePaused=0

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONUP, MOUSEMOTION):
                self.actives.update(event)
        if self.cap.running():
            self.cap.get_and_flip()
        
#                if self.moviePlaying:
#                    if self.moviePaused==1: self.unPauseMovie() 
#                    else: self.pauseMovie() 
#        return 
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010-2011 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           funcs.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os

if __name__ == '__main__':
    import sys
    sys.path.insert(0,'..')
import subprocess
import utils
import pygame
from pygame.constants import *
from SPConstants import TTF, NoGtk

def detect_keyboard():
    process = subprocess.Popen(['lsusb -v | grep "Keyboard"'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    txt = list(process.communicate())
    for line in txt[0].split('/n'):
        if "Keyboard" in line or "keyboard" in line:
            return True
    return False


def make_button_bg_dynamic(width, sizename, colorname, colorname_ro='black', THEME={}):
    """width - integer for the total length of the button images.
    sizename - name for the kind of image set to use.
    supported size names are: '36px','54px','81px'.
    colorname - name for the color to use.
    colorname_ro - name for the 'rollover' color to use.
    Supported colornames are: 'black', 'blue', 'green'.
    When a name isn't found a Exception is raised."""
    try:
        left = utils.load_image(os.path.join(THEME['themepath'], "%s_%s_%s.png" % (sizename, 'left', colorname)))
        right = utils.load_image(os.path.join(THEME['themepath'], "%s_%s_%s.png" % (sizename, 'right', colorname)))
        center = utils.load_image(os.path.join(THEME['themepath'], "%s_%s_%s.png" % (sizename, 'center', colorname)))
        hleft = utils.load_image(os.path.join(THEME['themepath'], "%s_%s_%s.png" % (sizename, 'left', colorname_ro)))
        hright = utils.load_image(os.path.join(THEME['themepath'], "%s_%s_%s.png" % (sizename, 'right', colorname_ro)))
        hcenter = utils.load_image(os.path.join(THEME['themepath'], "%s_%s_%s.png" % (sizename, 'center', colorname_ro)))
    except IOError, info:
        raise Exception, info
    partwidth = left.get_rect().w 
    hight = left.get_rect().h
    x = width - partwidth*2
    if x < 1:
        x = 1
        width = partwidth *2 +1
    
    but = pygame.Surface((4+width, 4+hight), SRCALPHA)
    buthov = pygame.Surface((4+width, 4+hight), SRCALPHA)
    newcenter = pygame.transform.scale(center, (x, center.get_rect().h))
    hnewcenter = pygame.transform.scale(hcenter, (x, center.get_rect().h))
    but.blit(newcenter, (partwidth, 2))
    buthov.blit(hnewcenter, (partwidth, 2))
    partwidth += newcenter.get_rect().w
    
    but.blit(left, (2, 2))
    buthov.blit(hleft, (2, 2))

    but.blit(right, (partwidth, 2))
    buthov.blit(hright, (partwidth, 2))
    return (but, buthov)

def make_dialog_bg_dynamic(width, height, THEME):
    sizes = {}
    try:
        topleft = utils.load_image(os.path.join(THEME['themepath'], "dialog_top_left.png" ))
        topright = utils.load_image(os.path.join(THEME['themepath'], "dialog_top_right.png" ))
        topcenter = utils.load_image(os.path.join(THEME['themepath'], "dialog_top_center.png" ))
        botleft = utils.load_image(os.path.join(THEME['themepath'], "dialog_bottom_left.png" ))
        botright = utils.load_image(os.path.join(THEME['themepath'], "dialog_bottom_right.png" ))
        botcenter = utils.load_image(os.path.join(THEME['themepath'], "dialog_bottom_center.png" ))
        rightcenter = utils.load_image(os.path.join(THEME['themepath'], "dialog_center_right.png" ))
        leftcenter = utils.load_image(os.path.join(THEME['themepath'], "dialog_center_left.png" ))
        centcenter = utils.load_image(os.path.join(THEME['themepath'], "dialog_center_center.png" ))
    except IOError, info:
        raise Exception, info
    dlg = pygame.Surface((4+width, 4+height), SRCALPHA)
    partwidth = topleft.get_rect().w 
    partheight = topleft.get_rect().h
    y = 0
    centerwidth = width - partwidth * 2
    centerheight = height - partheight * 2
    dlg.blit(topleft,(0,y))
    dlg.blit(pygame.transform.scale(topcenter,(centerwidth,partheight)), (partwidth,y))
    dlg.blit(topright,(partwidth + centerwidth, y))
    y += partheight
    sizes['title_area'] = pygame.Rect(0,0,centerwidth + partwidth * 2,y)
    temp_y = y       
    dlg.blit(pygame.transform.scale(leftcenter,(partwidth,centerheight)), (0,y))
    dlg.blit(pygame.transform.scale(centcenter,(centerwidth,centerheight)), (partwidth,y))
    dlg.blit(pygame.transform.scale(rightcenter,(partwidth,centerheight)), (partwidth + centerwidth,y))
    y += centerheight
    sizes['action_area'] = pygame.Rect(0,temp_y,centerwidth + partwidth * 2,y)
    
    partheight = botleft.get_rect().h
    dlg.blit(botleft,(0,y))
    dlg.blit(pygame.transform.scale(botcenter,(centerwidth,partheight)), (partwidth,y))
    dlg.blit(botright,(partwidth + centerwidth, y))
    sizes['button_area'] = pygame.Rect(0,y,centerwidth + partwidth * 2, y +partheight )
    return (dlg, sizes)

def grey_it(img):
    red       = False
    green     = True
    blue      = False
    for x in range(img.get_width()):
        for y in range(img.get_height()):
            c = list(img.get_at((x,y))[:3])
            deg = (c[0]*red + c[1]*green + c[2]*blue) / (red + green + blue)
            img.fill( [deg]*3, ((x,y),(1,1)) )
    return img

def get_boxes(color, hover_color, rect, selected_color=None, invert=False):
    """Draws rectangles for the buttons and dialogs."""
    s = pygame.Surface((rect.w, rect.h))
    s_hover = s.convert()
    pygame.draw.rect(s, color, rect, 0)
    hcolor = []
    rect.move_ip(-1, -1)
    for c in color:
        if c > 80:
            c -= 70
        elif c < 180:
            c +=70
        hcolor.append(c)
    h_color = tuple(hcolor)
    pygame.draw.rect(s, h_color, rect, 2)
    
    s_hover.fill(hover_color)
    hcolor = []
    for c in color:
        if c > 80:
            c -= 70
        elif c < 180:
            c += 70
        hcolor.append(c)
    h_color = tuple(hcolor)
    pygame.draw.rect(s_hover, hcolor, rect, 2)
    if selected_color:
        hcolor = []
        s_select = s_hover.convert()
        s_select.fill(selected_color)
        for c in color:
            if c > 80:
                c -= 70
            elif c < 180:
                c += 70
            hcolor.append(c)
        h_color = tuple(hcolor)
        pygame.draw.rect(s_select, hcolor, rect, 2)
        return (s, s_hover, s_select)
    return (s, s_hover)

def render_textrect(string, fsize, family, rect, text_color, background_color,\
                    justification=0, autofit=False, border=False, padding=8, bold=False):
    """Returns a surface containing the passed text string, reformatted
    to fit within the given rect, word-wrapping as necessary. The text
    will be anti-aliased.

    Takes the following arguments:

    string - the text you wish to render. \n begins a new line.
    font - a Font object
    rect - a rectstyle giving the size of the surface requested.
            If a pygame.Surface is given iso a rect the surface is used to blit the text to.
    text_color - a three-byte tuple of the rgb value of the
                 text color. ex (0, 0, 0) = BLACK
    background_color - a three-byte tuple of the rgb value of the surface.
    justification - 0 (default) left-justifiedSPWidgets.py
                    1 horizontally centered
                    2 right-justified
    autofit - When True any space left at the bottom of the surface will be discarded.
    border - A border is drawn around the surface.blit
    padding - Space around the text.
    Returns the following values:
    Success - a surface object with the text rendered onto it.
    Failure - raises a TextRectException if the text won't fit onto the surface.
    """
    final_lines = []
    requested_lines = string.splitlines()
    if NoGtk:
        font = pygame.font.Font(TTF, fsize)
    else:
        font = utils.pangofont.PangoFont(family=family, size=fsize, bold=bold)
    if type(rect) is not pygame.Rect:
        # we assume a surface
        if rect.get_alpha():
            surface = rect.convert_alpha()
        else:
            surface = rect.convert()
        rect = rect.get_rect()
    else:
        if background_color == 'trans':
            surface = pygame.Surface(rect.size, SRCALPHA)
            surface.fill((0, 0, 0, 0))
        else:
            surface = pygame.Surface(rect.size)
            surface.fill(background_color)
    # Create a series of lines that will fit on the provided
    # rectangle.
    # we keep an padding on both sides
    for requested_line in requested_lines:
        if font.size(requested_line)[0] > (rect.width - padding*2):
            words = requested_line.split(' ')
            # if any of our words are too long to fit, return.
#            for word in words:
#                if font.size(word)[0] >= rect.width-16:
#                    raise TextRectException, "The word " + word + " is too long to fit in the rect passed."
            # Start a new line
            accumulated_line = ""
            for word in words:
                test_line = accumulated_line + word + " "
                # Build the line while the words fit.    
                if font.size(test_line)[0] < rect.width-padding*2:
                    accumulated_line = test_line
                else:
                    final_lines.append(accumulated_line)
                    accumulated_line = word + " "
            final_lines.append(accumulated_line)
        else:
            final_lines.append(requested_line)

    accumulated_height = 0
    for line in final_lines:
#        if accumulated_height + font.size(line)[1] >= rect.height:
#            raise TextRectException, "Once word-wrapped, the text string was too tall to fit in the rect."
        if line != "":
            tempsurface = font.render(line, 1, text_color)
            if justification == 0:
                surface.blit(tempsurface, (8, accumulated_height+8))
            elif justification == 1:
                surface.blit(tempsurface, ((rect.width-16 - tempsurface.get_width()) / 2, accumulated_height+8))
            elif justification == 2:
                surface.blit(tempsurface, (rect.width-16 - tempsurface.get_width(), accumulated_height+8))
#            else:
#                raise TextRectException, "Invalid justification argument: " + str(justification)
        accumulated_height += font.size(line)[1]
    if autofit:
        y = accumulated_height+16
        if y > 500:
            y = 500
        if background_color == 'trans':
            s = pygame.Surface((rect.w, y), SRCALPHA)
            s.fill((0, 0, 0, 0))
        else:
            s = pygame.Surface((rect.w, y))
            s.fill(background_color)
        s.blit(surface, (0, 0))
        surface = s
    if border:
        pygame.draw.rect(surface, (0, 0, 0), surface.get_rect().inflate(-1, -1), 1)
    return surface

if __name__ == '__main__':
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    
    from base import Init
        
    import pygame
    from pygame.constants import *
    pygame.init()
    
    from SPSpriteUtils import SPInit
    
    scr = pygame.display.set_mode((800, 600))
    scr.fill((240,240,250))
    pygame.display.flip()
    back = scr.convert()
    actives = SPInit(scr, back)
    
    THEME = Init('braintrainer')
    dlg = make_dialog_bg_dynamic(300,500,THEME)
    
    scr.blit(dlg,(100,100))
    pygame.display.update()
    runloop = 1 
    while runloop:
        pygame.time.wait(100)
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type is KEYDOWN:
                if event.key == K_ESCAPE:
                    runloop = 0
    
    
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
# Copyright (c) 2006 Miguel Pérez Francisco <mperez@icc.uji.es>
#
#           puzzle.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.puzzle.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.puzzle")

# standard modules you probably need
import os,sys,random,glob, textwrap

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets

########################################
# Constats definition
########################################

# Position of the splitted image
INIX=20
INIY=0

# Position of the target image
INIXF=450
INIYF=0

# Offset between the position of the mouse and the piece being translated
OFFSETIMGX=30
OFFSETIMGY=30

# Size of the lines between pieces
TAMLINEA=2

#Position of the thumbnail of the image
INIXTH,INIYTH=350,396

# containers that can be used globally to store stuff
class Img:
    pass
class Snd:
    pass
class Misc:
    score = 0

#TODO: make sure we don't get the same image twice in one level

class Piece(SPSpriteUtils.SPSprite):
    """ A piece of a puzzle """
    Selected = None
    Brothers = []
    def __init__(self,image, correct_position, obs, name):
        self.org_image = image.convert()
        shadow = pygame.Surface(image.get_rect().size)
        shadow.fill(DARKGREEN)
        self.shadow = SPSpriteUtils.SPSprite(shadow)
        pygame.draw.rect(image, BLACK, image.get_rect(), TAMLINEA)
        SPSpriteUtils.SPSprite.__init__(self,image)
        self.connect_callback(self._activated_cbf, MOUSEBUTTONDOWN)
        self.correct_position = correct_position
        self.name = name
        self.logger = logging.getLogger("childsplay.puzzle.Piece")
        self.ImSelected = False
        self.obs = obs
        Piece.Brothers.append(self) 
        self.temp = 140
        
    def _activated_cbf(self, sprite, event, data):
        #self.logger.debug("_activated_cbf called")
        if Piece.Selected and self.name == Piece.Selected.name:
            self._do_unselect()
        elif self.name == 'piece' and not Piece.Selected:
            self._do_select()
        elif self.name == 'target' and Piece.Selected:
            self._do_check()
                    
    def _do_select(self):
        if self.ImSelected:
            return
        #self.logger.debug("_do_select called")
        self.ImSelected = True
        Piece.Selected = self
        # we are placed at a position by the caller
        self.org_position = self.rect.topleft
        x, y = self.rect.topleft
        self.erase_sprite()
        self.shadow.moveto((x+24, y+20))
        self.moveto((x+12, y+12))
        for s in Piece.Brothers:
            s.do_fade()
        self.shadow.display_sprite()
        self.display_sprite()
        
    def _do_unselect(self):
        if not self.ImSelected:
            return
        #self.logger.debug("_do_unselect called")
        self.ImSelected = False
        Piece.Selected.moveto(self.org_position, hide=False)
        Piece.Selected.shadow.erase_sprite()
        Piece.Selected = None
        for s in self.group_owner:
            s.image.set_alpha(255)
            s.display_sprite()
        
        
    def _do_check(self):
        #self.logger.debug("_do_check called")
        if self.correct_position == Piece.Selected.correct_position:
            #self.logger.debug('correct piece selected')
            Piece.Selected.image = Piece.Selected.org_image
            Piece.Selected.moveto(self.rect.topleft, hide=False)
            Piece.Selected.kill()
            self.kill()
            Piece.Selected.shadow.erase_sprite()
            Piece.Selected = None
            for s in self.group_owner:
                s.image.set_alpha(255)
                s.display_sprite()
            self.obs(1)
        else:
            self.obs(-1)
   
    def do_fade(self):
        """make the image a bit more transparent to let the selected piece standout more"""
        if self.ImSelected or self.name == 'target':
            return
        self.erase_sprite()
        self.image.set_alpha(80)
        self.display_sprite()
            
class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activity name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.puzzle.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','PuzzleData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'puzzle.rc'))
        self.rchash['theme'] = self.theme
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        self.DbAssets = os.path.join(self.SPG.get_libdir_path(),'CPData', 'DbaseAssets')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        self.actives.set_onematch(True)
        imgdir = os.path.join(self.my_datadir, self.theme)
        if not os.path.exists(imgdir):
            old = imgdir
            imgdir = os.path.join(self.my_datadir,'childsplay')
            self.logger.debug("not found %s using %s" % (old, imgdir))
        target_path = os.path.join(imgdir,'target.png')
        self.target_image = utils.load_image(target_path)
       
        self.gamelevels = [(2,2,10),
                           (3,2,10),
                           (3,3,10),
                           (4,3,10),
                           (4,4,10)]
        # Images must be 324x324
        # Image Size, by now these values has to be "divisibles" by 2,3,4,5,6
        self.xsize = 324
        self.ysize = 324
        
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Puzzle")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "puzzle"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("Place the puzzle piece on the right where you think it belongs..."), 
        " ",
        _("At the bottom you'll see a image of the complete puzzle."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return _("Look closely at the complete puzzle picture at the bottom.")
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Puzzle")
    
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 5

    
    def start(self):
        """Mandatory method."""
        # Your top blit position, this depends on the menubar position 
        self.tile = 'tileset_1'
        if self.blit_pos[1] == 0:
            self.blit_pos_y = 10
        else:
            self.blit_pos_y = 110
        self.SPG.tellcore_set_dice_minimal_level(2)
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.ThumbsUp = SPSpriteUtils.MySprite(utils.load_image(p))
        self.good = utils.load_music(os.path.join(self.CPdatadir, 'good1.ogg'))
        self.wrong = utils.load_music(os.path.join(self.CPdatadir, 'wrong1.ogg'))
        self.score = 0
        self.AreWeDT = False
        self.pre_level_flag = False
     
    def pre_level(self, level):
        """Mandatory method"""
        self.logger.debug("pre_level called with: %s" % level)
        if level > len(self.gamelevels):
            return
        self.pre_level_flag = True
        y = self.blit_pos_y
        
        imgdir = os.path.join(self.my_datadir, self.theme)
        if not os.path.exists(imgdir):
            imgdir = os.path.join(self.my_datadir,'childsplay')
        
        files = glob.glob(os.path.join(imgdir,'tileset_*.png'))
        files.sort()
        lbl = SPWidgets.Label(_("Please, choose a set of puzzle images."),\
                                (30, y),fsize=24, transparent=True)
        lbl.display_sprite()
        y += lbl.get_sprite_height()
        for line in textwrap.wrap(_("The difficulty of the set increases from left to right."), 40):
            lbl = SPWidgets.Label(line,(30, y), fsize=24, transparent=True)
            lbl.display_sprite()
            y += lbl.get_sprite_height()
        y += 50
        tmp_img = utils.load_image(files[0])
        count = 1
        total_width = ((len(files) / 2) * (tmp_img.get_width())) + (((len(files) / 2) - 1)  * 20)
        first_x = (self.screen.get_width() / 2) - (total_width / 2)
        for tilepath in files:
            x = first_x + (count-1)*tmp_img.get_width() + (count-1)*20
            img = utils.load_image(tilepath)
            data = os.path.splitext(os.path.split(tilepath)[1])[0]
            if not data[-2:] == 'ro':
                img_ro = utils.load_image(os.path.join(imgdir, '%s_ro.png' % data))
                b = SPWidgets.SimpleTransImgButton(img, img_ro, (x, y), data=data)
                b.display_sprite()
                self.actives.add(b)
                count += 1
        return True
  
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        if level > len(self.gamelevels):
            return
        self.level = level
        self.levelupcount = 1
        self.dbscore = 0
        Piece.Brothers = []
        # number of exercises in one level
        if self.AreWeDT:
            self.exercises = 1
        else:
            self.exercises = int(self.rchash[self.theme]['exercises'])
        
        self.totalexercises = self.exercises
        self.xoffset = 420 # space between the two puzzle areas left borders
        self.db_mapper = dbmapper
        self.rows, self.cols ,self.bonus = self.gamelevels[level-1]
        self.npieces=self.cols*self.rows
        imgdir = os.path.join(self.my_datadir, self.theme, self.tile)
        if not os.path.exists(imgdir):
            old = imgdir
            imgdir = os.path.join(self.my_datadir,'childsplay', self.tile)
            self.logger.debug("not found %s using %s" % (old, imgdir))
        self.logger.debug("Using imagedir %s" % imgdir)
        self.pzimages = glob.glob(os.path.join(imgdir,'*.jpg'))
        random.shuffle(self.pzimages)
        self.logger.debug("Found %s images" % len(self.pzimages))
        # store number of cards into the db table 'cards' col
        self.db_mapper.insert('total_pieces',self.npieces)
        # bonusp is the score for each piece put in its place
        self.bonusp=self.bonus/self.npieces
        # size of a piece
        self.incx=self.xsize/self.cols
        self.incy=self.ysize/self.rows
        self.clear_screen()
        self.next_exercise()

        return True
    
    
    
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        # TODO: it's hardcoded, for 2.2 we must have it in the rcfile
        self.tile = 'tileset_1'

    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
        # M = Maximum score (number of pieces * total images)
        # P = Users result in points
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        M = float(self.npieces * self.totalexercises)
        P = float(self.dbscore)
        F = 7.0
        F1 = 3.0
        C = 0.2
        S = float(seconds)
        points = max((F/100) * (P/(M / 100)), 1.0)
        time = min(F1 / ((S / (M * 2)) ** C), F1)
        self.logger.info("@scoredata@ %s level %s M %s P %s S %s points %s time %s" % \
                         (self.get_name(),self.level, M, P, S, points, time))
        score = points + time
        result = points + time
        return result
        
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def score_observer(self, score):
        if score < 0:
            self.wrong.play()
        else:
            self.good.play()
        self.score += score * 10
        self.dbscore += score
        if self.score <= 0:
            self.score = 10
        if score > 0:
            # correct answer
            self.levelupcount += 1
        else:
            self.levelupcount -= 1

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP, MOUSEMOTION):
                result = self.actives.update(event)
                if self.pre_level_flag:
                    if result:
                        self.logger.debug("loop, got a result and stop this loop")
                        self.SPG.tellcore_pre_level_end()
                        self.pre_level_flag = False
                        self.tile = result[0][1][0]
                        break
                if not self.actives:
                    self.scoredisplay.set_score(self.score*3)
                    pygame.time.wait(int(self.rchash[self.theme]['wait_value']))
                    self.clear_screen()
                    self.ThumbsUp.display_sprite((229, 120))
                    pygame.time.wait(1000)
                    self.ThumbsUp.erase_sprite()
                    if self.next_exercise():
                        self.logger.debug("%s" % self.level)
                        if self.level<5 and self.levelupcount >= int(self.rchash[self.theme]['autolevel_value']):
                            levelup = 1
                        else:
                            levelup = 0
                        if self.AreWeDT:
                            self.SPG.tellcore_level_end(level=self.level)
                        else:
                            self.SPG.tellcore_level_end(store_db=True, \
                                                level=min(6, self.level), \
                                                levelup=levelup)
        return 
     
    def next_exercise(self):
        self.logger.debug("next_exercise called, exercises = %s" % (self.exercises))
        
        self.actives.purge()# remove sprites from previous exercise and erase them
        if not self.exercises:
            return True
        else:
            self.load_puzzle()
            self.ok=[0]*len(self.actives)
            for s in self.actives:
                s.display_sprite()
            pygame.display.update()
            self.puzzlethumb.display_sprite((INIXTH-20,INIYTH+self.blit_pos_y-50))
            self.exercises -= 1
  
    def load_puzzle(self):
        """  Load an image to split, it choose one between the images in
         myimages folder and the childsplay images.
        """
        imgpath = self.pzimages.pop()
        image=utils.load_image(imgpath)
        img = utils.aspect_scale(image.convert(),(140,140))
        self.puzzlethumb = SPSpriteUtils.MySprite(img)
        positions=[] 
        self.pieceslist = []
                
        # Split the surfaces in pieces
        sc = 324 / (self.npieces/2)
        target_image = utils.aspect_scale(self.target_image, (sc, sc))
        ix=0
        px=self.incx*ix
        while px<self.xsize:
            iy=0
            py=self.incy*iy
            while py<self.ysize:
                position = (INIX+px,self.blit_pos_y+INIY+py)
                positions.append(position)
                r = pygame.Rect((px,py),(self.incx,self.incy))
                try:
                    piece=image.subsurface(r)
                except ValueError:
                    self.logger.debug("Failed to get a subsurface, image has wrong size %s" % imgpath)
                piece=Piece(piece, position,self.score_observer, name='piece')
                self.pieceslist.append(piece)
                s = pygame.Surface((r.w, r.h))
                s.fill(WHITE)
                s.blit(target_image, ((r.w - target_image.get_rect().w) / 2, \
                                       (r.h - target_image.get_rect().h) / 2))
                target = Piece(s, position, self.score_observer, name='target')
                target.moveto((position[0]+self.xoffset, position[1]))
                self.actives.add(target)
                iy=iy+1
                py=self.incy*iy
            ix=ix+1
            px=self.incx*ix
        
        self.org_positions=positions[:]
        while positions == self.org_positions:
            random.shuffle(positions)
            
        for s in self.pieceslist:
            s.moveto(positions.pop(0))
        self.actives.add(self.pieceslist)
        
        return

    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010 Stas Zykiewicz <stas.zytkiewicz@gmail.com>
#
#           SPGuiDBModel.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

""" This module provides an communication layer between a GUI and the SP dbase.
 GUI's should use this an not access the SP dbase stuff directly.
 This module assumes the GUI is used independant of SP.
 Use the 'get_datamanager' function to get a mapper object.
"""

import atexit,sys
sys.path.insert(0, 'gui')
#create logger, logger was configured in SPLogging
import logging
module_logger = logging.getLogger("schoolsplay.SPGuiDBModel")

# first parse commandline options, it's fake ofcourse as we are run from inside
# a python module but we need the options object

from SPOptionParser import OParser

op = OParser()
# this will return a class object with the options as attributes  
CMD_Options = op.get_options()

class SPGoodiesWrapper:
    """Wrapper class, needed to run the datamanager"""
    _cmd_options = CMD_Options  

try:
    import sqlalchemy as sqla
except ImportError:
    module_logger.exception("No sqlalchemy package found")
    raise Exception
else:
    if sqla.__version__ < '0.4':
        module_logger.error("Found sqlalchemy version %s" % sqla.__version__ )
        module_logger.error("Your version of sqlalchemy is to old, please upgrade to version >= 0.4")
    module_logger.debug("using sqlalchemy %s" % sqla.__version__)

import SPDataManager
from SPDataManagerCreateDbase import DbaseMaker
DEBUG = False
import SQLTables
from SPConstants import DBASEPATH
import utils

try:
    import sqlalchemy.exceptions as sqlae
except ImportError:
    from sqlalchemy import exc as sqlae

import sqlalchemy.orm as sqlorm
# get a datamanager
def get_datamanager():
    """use this to get a datamanager instance which could be used by a GUI.
    Typical usage would be:
        import SPGuiDBModel
        dm = SPGuiDBModel.get_datamanager()
        tablemapper = dm.get_mapper('<table name>')
        tabledata = tablemapper.get_table_data()
        # inserting is related to the table design
        newrow = [{id:<int>,'logon':<string>,'name':<string>}]
        tablemapper.update_row(data)
        """
    return DataManagerWrapper(SPGoodiesWrapper())

class DataManagerWrapper(SPDataManager.DataManager):
    """Wrapper class to set up a DataManager which can be used as a standalone.
    This is meant for GUI's that want to interface with the SP dbase.
    Don't use this class directly, use the 'get_datamanager' function."""
    def __init__(self,spgoodies):
        self.logger = logging.getLogger("schoolsplay.SPGuiDBModel.DataManagerWrapper")
        self.logger.debug("Starting")
        self.current_user_id = None # needed for the testsuite
        self.spg = spgoodies
        self.cmd_options = self.spg._cmd_options
        atexit.register(self._cleanup)
        
        try:
            dbm = DbaseMaker(self.cmd_options.theme, debug_sql=DEBUG)            
        except (AttributeError, sqlae.SQLAlchemyError, utils.MyError), info:
            self.logger.exception("Failed to start the DBase, %s" % info)
            raise MyError, info
        self.content_engine, self.user_engine = dbm.get_engines()
        self.metadata_contentdb, self.metadata_usersdb = dbm.get_metadatas()
        self.all_orms = dbm.get_all_orms()
        self.orms_content_db, self.orms_userdb = dbm.get_orms()
        self.UserSession = sqlorm.sessionmaker(bind=self.user_engine)
        self.ContentSession = sqlorm.sessionmaker(bind=self.content_engine)
        
               
    def get_mapper(self,activity):
        """Overrides the get_mapper from SPDataManager.
        We don't have or use user id's"""
        try:
            mclass = SPDataManager.RowMapper(self.tables[activity])
        except (sqla.exceptions.SQLError,KeyError):
            self.logger.exception("Failed to get mapper: %s" % activity)
            raise utils.NoSuchTable("Failed to get mapper: %s" % activity)
        else:
            return mclass
            
        
        
        
        



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
# Copyright (c) 2004  Matias Grana matiasg@dm.uba.ar
#           billiard.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.billiard.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.billiard")

# standard modules you probably need
import os, sys, operator, random
from math import sqrt, acos

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

# containers that can be used globally to store stuff
class Img:
    pass


class Snd:
    pass


class Misc:
    pass


class Hole:
    pass


class Ball(SPSpriteUtils.SPSprite):
    """
    A ball that can move on the table, collide with other ball and enter the hole.
    """

    def __init__(self, pic, start, kind):
        self.image = pic
        SPSpriteUtils.SPSprite.__init__(self, self.image,
                                        name=kind)  # embed it in this class, eg make SPSpriteUtils.SPSprite part of Ball
        self.rect = self.posint = pic.get_rect().move(start)
        self.posreal = (self.posint[0], self.posint[1])
        self.size = (self.posint.right - self.posint.left, self.posint.bottom - self.posint.top)
        self.sqradius = (self.size[1] / 2) ** 2
        self.smthns = .95  #between 0 and 1. The higher, the more the ball will roll
        self.direc = (0, 0)
        self.thkns = .9  #transmission of speed in collision

    def on_update(self, *args):
        """ This is called on every refresh call."""
        v = self.madehole()
        return v  # we use this to return 1 to the refresh call

    def moveit(self):
        """ Moves the ball """
        # It is important that the position be a real number. Otherwise the movement is not nice.
        # We cast it to int to blit it.
        self.posreal = (self.posreal[0] + self.direc[0], self.posreal[1] + self.direc[1])
        self.posint.left, self.posint.top = int(self.posreal[0]), int(self.posreal[1])
        # Don't fall out of the table!
        if self.posint.right > 792:
            self.posint.left = 792 - self.size[0]
            self.posreal = (self.posint.left, self.posreal[1])
            self.direc = (-self.direc[0], self.direc[1])
        if self.posint.bottom > 592:
            self.posint.top = 592 - self.size[1]
            self.posreal = (self.posreal[0], self.posint.top)
            self.direc = (self.direc[0], -self.direc[1])
        if self.posint.left < 8:
            self.posint.left = 8
            self.posreal = (8, self.posreal[1])
            self.direc = (-self.direc[0], self.direc[1])
        if self.posint.top < 108:
            self.posint.top = 108
            self.posreal = (self.posreal[0], 108)
            self.direc = (self.direc[0], -self.direc[1])
        # Slow down a bit!
        self.direc = (self.direc[0] * self.smthns, self.direc[1] * self.smthns)
        # If it's almost stopped, just stop it.
        if self.direc[0] ** 2 + self.direc[1] ** 2 < 1:
            self.direc = (0, 0)
            return 0
        return 1

    def addtodir(self, x, y):
        """ changes the direction of the ball """
        self.direc = (self.direc[0] + x, self.direc[1] + y)
        return self.direc[0] ** 2 + self.direc[1] ** 2

    def addtopos(self, x, y):
        """ changes position of the ball """
        self.posreal = (self.posreal[0] + x, self.posreal[1] + y)
        self.posint.left += x
        self.posint.top += y

    def callback(self, mousepos, mousebut):
        """ What to do under mouse button pressed.
         This is not a callback like the ones used in a typical SPSpriteUtils.SPSprite object."""
        diff = self.posint.move(self.size[0] / 2 - mousepos[0], self.size[1] / 2 - mousepos[1])
        dist = sqrt(diff[0] ** 2 + diff[1] ** 2)
        # see if the kick was inside the ball and out from the center. If yes, change direction.
        if (0 < dist <= self.size[0] / 2):
            valtoreturn = 1
            if (mousebut == 1):
                self.direc = (diff[0] / dist, diff[1] / dist)
        #utils.load_music(Snd.throw).play()
        else:
            valtoreturn = 0
        return valtoreturn

    def checkcollision(self, otherobj):
        """ checks whether two objects collide """
        x, y = (self.posreal[0] - otherobj.posreal[0], self.posreal[1] - otherobj.posreal[1])
        squaredist = x ** 2 + y ** 2
        dist = sqrt(squaredist)
        # check if both objects collide. If yes, change their directions.
        overlap = dist - (self.size[0] + otherobj.size[0]) / 2
        if overlap < 0:
            diffvel = (self.direc[0] - otherobj.direc[0], self.direc[1] - otherobj.direc[1])
            coef = self.thkns * (x * diffvel[0] + y * diffvel[1]) / squaredist
            newvel = self.addtodir(-x * coef, -y * coef)
            self.addtopos(-x * overlap / dist, -y * overlap / dist)
            self.moveit()
            newvelotherobj = otherobj.addtodir(x * coef, y * coef)
            otherobj.moveit()
            return (1, newvel, newvelotherobj)
        return (0,)

    def madehole(self):
        """ checks if ball entered the hole """
        diff = (self.posreal[0] - Hole.pos[0] + 30, self.posreal[1] - Hole.pos[1] + 30)
        sqdist = diff[0] ** 2 + diff[1] ** 2
        # this checks whether d(b,h) < radius  (b = center of ball, h = center of hole)
        if sqdist < self.sqradius:
            return (self, 1)  # self is the object
        return 0


class Stick(SPSpriteUtils.SPSprite):
    def __init__(self, pic, start):
        self.image = self.origpic = pic
        SPSpriteUtils.SPSprite.__init__(self, self.image)
        self.origsize = pic.get_rect()
        self.angle = 0
        self.start = start
        self.rect = self.posint = pic.get_rect().move(start)
        self.disp = [0, 0]
        self.maxstrength = 80  # the maximum force to hit the ball

    def prepare(self, posmouse, posball):
        self.positmouse = posmouse
        dif = (posball[0] - posmouse[0], posball[1] - posmouse[1])
        dist = sqrt(dif[0] ** 2 + dif[1] ** 2)
        dif = (dif[0] / dist, dif[1] / dist)
        self.angle = acos(dif[0]) * 180 / 3.141593
        if dif[1] > 0:
            self.angle = 360 - self.angle
        self.image = self.pic = pygame.transform.rotate(self.origpic, self.angle)
        if dif[0] > 0:
            self.disp[0] = -self.origsize.right * dif[0]
        else:
            self.disp[0] = 0
        if dif[1] > 0:
            self.disp[1] = -self.origsize.right * dif[1]
        else:
            self.disp[1] = 0
        self.displacement = (posmouse[0] + self.disp[0], posmouse[1] + self.disp[1])
        self.rect = self.posint = self.pic.get_rect().move(self.displacement)

    def update(self, *args):
        pass

    def grow(self, scale):
        # make the stick grow to indicate strength
        sc = (float(scale) / self.maxstrength) ** 2 + 1
        self.image = pygame.transform.scale(self.pic, (int(self.posint.width * sc), int(self.posint.height * sc) ))
        self.rect = self.pic.get_rect().move(self.positmouse[0] + int(self.disp[0] * sc),
                                             self.positmouse[1] + int(self.disp[1] * sc))
        Misc.group.refresh()

    def draw(self, screen):
        screen.blit(self.pic, self.posint)


class MovingSign(SPSpriteUtils.SPSprite):
    def __init__(self, txt, size, start, stop, step, fcol, scr, bck, era):
        self.image, self.size = utils.text2surf(txt, size, fcol, bold=True)
        SPSpriteUtils.SPSprite.__init__(self, self.image)
        self.rect = self.image.get_rect()
        self.set_movement(start, stop, step, 1, 0)
        self.add(Misc.movingsign)
        self.display_sprite()
        self.wait = max(abs(stop[0] - start[0]), abs(stop[1] - start[1]))
        while self.wait:
            self.wait -= step
            Misc.movingsign.refresh()
            pygame.time.wait(5)
        self.remove(Misc.movingsign)
        if era:
            Misc.movingsign.refresh()


class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self, SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger = logging.getLogger("childsplay.billiard.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size)  # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(), 'CPData', 'BilliardData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(), 'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()  # alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'c'))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup

        Misc.actives = SPSpriteUtils.SPInit(self.screen, self.backgr)
        Misc.movingsign = SPSpriteUtils.SPGroup(self.screen, self.backgr)
        Misc.group = SPSpriteUtils.SPGroup(self.screen, self.backgr)
        # MovingSign controls itself
        MovingSign(_("Billiard Game"), 48, (100, 100), (100, 400), 1, (8, 0, 120), self.screen, self.backgr, 1)

    def clear_screen(self):
        self.screen.blit(self.orgscreen, self.blit_pos)
        self.backgr.blit(self.orgscreen, self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        Misc.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Billiard")

    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "billiard"

    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("You have to make the blue ball enter the hole (in level 1)"),
                _("and the red ones in levels 2 and 3."),
                _("Use the right mousebutton to aim and the left button to hit the ball."),
                _("The longer you hold the left button the harder it will hit the ball."),
                _("The fewer hits you need to get the ball in the hole, the more points you get."),
        ]
        return text

    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return [_("The fewer hits you need to get the ball in the hole, the more points you get.")]

    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Fun/Miscellaneous")


    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 3

    def pre_level(self, level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass

    def next_level(self, level, dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        self.level = level  # make it part of this class so we can use it to test
        # in which level we are. See def _made_hole
        self.db_mapper = dbmapper
        self.points = 0
        self.tries = 0
        self.n_balls = 5
        self.maxpoints = 0
        # Tell the core to display a exercises/done counter
        Misc.execounter = self.SPG.tellcore_display_execounter(self.n_balls, text=_("Balls"))

        self.next_exercise()
        return True

    def start(self):
        """Mandatory method."""
        Hole.img = utils.load_image(os.path.join(self.my_datadir, 'hole.png'))
        Hole.size = (Hole.img.get_width(), Hole.img.get_height())

        self.backgr_wohole = utils.load_image(os.path.join(self.my_datadir, 'backgr.png'))
        Snd.throw = os.path.join(self.my_datadir, 'sndt.wav')
        Snd.hurra = os.path.join(self.my_datadir, 'sndh.wav')
        Snd.great = os.path.join(self.my_datadir, 'sndh.wav')

    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass

    def get_score(self, timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m, s = timespend.split(':')
        seconds = int(m) * 60 + int(s)
        if self.tries:
            score = max(1, 10 * (float(self.n_balls) / self.tries) - (seconds / 100.0))
        else:
            score = 0

        self.logger.debug('total time %s' % seconds)
        self.logger.debug('score %s' % score)
        if score == 0:
            return score
        # As this method is called by the core when a level is finished
        # we also use it to store the counters
        self.db_mapper.insert('balls', self.n_balls)
        #self.logger.debug("balls %s" % self.n_balls)
        self.db_mapper.insert('tries', self.tries)
        #self.logger.debug("tries %s" % self.tries)
        self.db_mapper.insert('maxpoints', self.maxpoints)
        #self.logger.debug("maxpoints %s" % self.maxpoints)
        self.db_mapper.insert('points', self.points)
        #self.logger.debug("points %s " % self.points)
        return score

    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method"""
        try:
            self.timer.stop()
        except:
            pass

    def loop(self, events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        if not self.buttonpressed:
            for event in events:
                if event.type is MOUSEBUTTONDOWN:
                    #if self.ball1 in Misc.actives.sprites():
                    if len(Misc.actives):
                        continue
                    if event.button >= 2:
                        self.logger.debug("button >=2 down")
                        if self.ball1.callback(event.pos, event.button):
                            # put stick in position to see the direction it will have
                            self.stick.prepare(event.pos, (self.ball1.posreal[0] + self.ball1.size[0] / 2,
                                                           self.ball1.posreal[1] + self.ball1.size[1] / 2))
                            Misc.group.add(self.stick)
                            Misc.group.refresh()
                    else:
                        self.logger.debug("button 1 down")
                        if self.ball1.callback(event.pos, event.button):
                            # there will be a kick. Begin counting the time buttonmouse is pressed.
                            self.buttonpressed = 1
                            self.timepressed = 0
                            self.stick.prepare(event.pos, (self.ball1.posreal[0] + self.ball1.size[0] / 2,
                                                           self.ball1.posreal[1] + self.ball1.size[1] / 2))
                            Misc.group.add(self.stick)
                            Misc.group.refresh()
                if event.type is KEYDOWN and (event.key == 113):
                    # My mind is going down...
                    #self.stop = 1
                    self.start(self.level, None)
                if event.type is MOUSEBUTTONUP:
                    Misc.group.remove(self.stick)
                    Misc.group.refresh()
        else:
            if len(events):
                for event in events:
                    if event.type is MOUSEBUTTONUP and event.button == 1:
                        # stick was released. It's time to roll.
                        self.stick.remove(Misc.group)
                        self.buttonpressed = 0
                        #Misc.group.refresh()
                        Misc.actives.add(self.ball1)
                        strength = self.timepressed / 2
                        self.ball1.direc = (self.ball1.direc[0] * strength, self.ball1.direc[1] * strength)
                        self.throws += 1
                        utils.load_music(Snd.throw).play()
            else:
                # strength is going up
                if self.timepressed < self.stick.maxstrength:
                    self.timepressed += 1
                    self.stick.grow(self.timepressed)
        if Misc.actives.sprites():
            #pygame.time.wait(10)
            self.movethings()
            # Call refresh with a event to let it call callback and update methods.
            # See the game fallingletters for a simple implementation of this in action.
            v = Misc.group.refresh(pygame.event.Event(USEREVENT))  # The ball returns 1 when it goes into the hole
            # the ball also removes it self from the grou. see the ball class

            # check the return val from the on_update method
            # for the format see the reference and the ball class
            if v:
                #print "return val from inside refresh:", v # just as info to see how the return val looks like :-)
                if v[0][1][1] == 1:
                    self._made_hole(v[0][1][0])
        return

    def next_exercise(self):
        self.logger.debug("next_exercise called")
        self.throws = 0
        self.buttonpressed = 0
        Misc.actives.empty()
        Misc.group.empty()
        Misc.score = 0  # see def loop and further
        objects = []
        self.screen.blit(self.backgr_wohole, (
        0, self.blit_pos[1]))  # this is needed here, as CP calls the start (and not setup) proc. on a level change.
        x, y = (random.randrange(8, 780 - Hole.size[0], 1),
                random.randrange(self.blit_pos[1] + 50, 600 - self.blit_pos[1] - Hole.size[1], 1))
        Hole.pos = (x, y)
        objects.append([Hole.pos, Hole.size])
        self.backgr.blit(self.backgr_wohole, (0, self.blit_pos[1]))
        self.screen.blit(self.backgr_wohole, (0, self.blit_pos[1]))
        self.screen.blit(Hole.img, Hole.pos)
        self.backgr.blit(Hole.img, Hole.pos)
        #self.screen.blit(self.backgr,(0,0))
        pygame.display.update()
        img = utils.load_image(os.path.join(self.my_datadir, 'ball1.png'))
        imgsize = (img.get_width(), img.get_height())
        x, y = (random.randrange(8, 780 - imgsize[0], 1), \
                random.randrange(self.blit_pos[1] + 50, 491 - self.blit_pos[1] - imgsize[1], 1))
        while self.tooclose((x, y, imgsize), objects):
            x, y = (random.randrange(8, 791 - imgsize[0], 1), \
                    random.randrange(self.blit_pos[1] + 50, 491 - self.blit_pos[1] - imgsize[1], 1))
        objects.append([(x, y), imgsize])
        self.ball1 = Ball(img, (x, y),
                          'blue')  # the extra argument can be 'blue' or 'red'. this way we know what kind of ball it is.
        # use this if you just want to display a sprite
        self.ball1.display_sprite()
        Misc.actives.add(self.ball1)
        Misc.group.add(self.ball1)
        print "level", self.level
        # Now it has level 2 repeated in level 3. Before it was (if level == 1), (if level > 1), (if level > 2).
        if self.level == 1:
            # set points to earn: just d(b1,h)/8 in this case
            self.pointstoadd = ( sqrt((self.ball1.posint[0] - Hole.pos[0]) ** 2 \
                                      + (self.ball1.posint[1] - Hole.pos[1]) ** 2) ) / 8
        if self.level == 2:
            img = utils.load_image(os.path.join(self.my_datadir, 'ball2.png'))
            imgsize = (img.get_width(), img.get_height())
            x, y = (random.randrange(8, 791 - imgsize[0], 1), random.randrange(8, 491 - imgsize[1], 1))
            while self.tooclose((x, y, imgsize), objects):
                x, y = (random.randrange(8, 791 - imgsize[0], 1), random.randrange(8, 491 - imgsize[1], 1))
            self.ball2 = Ball(img, (x, y), 'red')
            self.ball2.display_sprite()
            Misc.actives.add(self.ball2)
            Misc.group.add(self.ball2)
            # set points to earn: 0.2 * d(b2,h) + (2.4 + cos angle) * d(b2,b1) / 8
            # where b1,b2 = ball1,ball2, h = hole, d = distance, angle = angle at b2.
            diff1 = (Hole.pos[0] - self.ball2.posint[0], Hole.pos[1] - self.ball2.posint[1])
            diff2 = (self.ball1.posint[0] - self.ball2.posint[0], self.ball1.posint[1] - self.ball2.posint[1])
            diff1norm = sqrt(diff1[0] ** 2 + diff1[1] ** 2)
            diff2norm = sqrt(diff2[0] ** 2 + diff2[1] ** 2)
            self.pointstoadd = 0.2 * diff1norm + (diff1[0] * diff2[0] + diff1[1] * diff2[1]) / (
            6 * diff1norm) + diff2norm * 0.3
        if self.level == 3:
            img = utils.load_image(os.path.join(self.my_datadir, 'ball2.png'))
            imgsize = (img.get_width(), img.get_height())
            x, y = (random.randrange(8, 791 - imgsize[0], 1), random.randrange(8, 491 - imgsize[1], 1))
            while self.tooclose((x, y, imgsize), objects):
                x, y = (random.randrange(8, 791 - imgsize[0], 1), random.randrange(8, 491 - imgsize[1], 1))
            objects.append([(x, y), imgsize])
            self.ball2 = Ball(img, (x, y), 'red')
            x, y = (random.randrange(8, 791 - imgsize[0], 1), random.randrange(8, 491 - imgsize[1], 1))
            while self.tooclose((x, y, imgsize), objects):
                x, y = (random.randrange(8, 791 - imgsize[0], 1), random.randrange(8, 491 - imgsize[1], 1))
            self.ball3 = Ball(img, (x, y), 'red')
            self.ball2.display_sprite()
            self.ball3.display_sprite()
            Misc.actives.add((self.ball2, self.ball3))
            Misc.group.add((self.ball2, self.ball3))
            #points to earn: as in level 2  +  0.2 * d(b3,h) + (2.4 + cos angle) * d(b3,b1) / 8
            diff1 = (Hole.pos[0] - self.ball2.posint[0], Hole.pos[1] - self.ball2.posint[1])
            diff2 = (self.ball1.posint[0] - self.ball2.posint[0], self.ball1.posint[1] - self.ball2.posint[1])
            diff1norm = sqrt(diff1[0] ** 2 + diff1[1] ** 2)
            diff2norm = sqrt(diff2[0] ** 2 + diff2[1] ** 2)
            self.pointstoadd = 0.2 * diff1norm + (diff1[0] * diff2[0] + diff1[1] * diff2[1]) / (8 * diff1norm) + diff2norm * 0.3
            diff1 = (Hole.pos[0] - self.ball3.posint[0], Hole.pos[1] - self.ball3.posint[1])
            diff2 = (self.ball1.posint[0] - self.ball3.posint[0], self.ball1.posint[1] - self.ball3.posint[1])
            diff1norm = sqrt(diff1[0] ** 2 + diff1[1] ** 2)
            diff2norm = sqrt(diff2[0] ** 2 + diff2[1] ** 2)
            self.pointstoadd += 0.2 * diff1norm + (diff1[0] * diff2[0] + diff1[1] * diff2[1]) / (8 * diff1norm) + diff2norm * 0.3
        img = utils.load_image(os.path.join(self.my_datadir, 'stick.png'))
        self.stick = Stick(img, (200, 120))

    def tooclose(self, ob1, obs):
        for i in obs:
            if (((ob1[0] - i[0][0]) ** 2 + (ob1[1] - i[0][1]) ** 2) < (ob1[2][0] + i[1][0]) ** 2 / 4):
                return 1
        return 0

    def movethings(self):
        """ moves everything. Deletes from actives if stop. """
        for o in Misc.actives.sprites():
            keepit = o.moveit()
            # remove it from actives if it has stopped.
            if not keepit:
                Misc.actives.remove(o)
        # check all possible collisions
        for n1 in range(len(Misc.group.sprites())):
            o1 = Misc.group.sprites()[n1]
            for o2 in Misc.group.sprites()[n1:]:
                if o1 != o2:
                    newdirs = o1.checkcollision(o2)
                    if newdirs[0]:
                        if newdirs[1] == 0 and o1 in Misc.actives.sprites():
                            Misc.actives.remove(o1)
                        elif newdirs[1] and o1 not in Misc.actives.sprites():
                            Misc.actives.add(o1)
                        if newdirs[2] and o2 not in Misc.actives.sprites():
                            Misc.actives.add(o2)
                        elif newdirs[2] == 0 and o2 in Misc.actives.sprites():
                            Misc.actives.remove(o2)

    def _made_hole(self, obj):
        # Keeps also track of the score
        # This replaces _level_1,_level_2 and _level_3
        if self.level > 1 and obj == 'blue':
            return  # we pot a red blue ball in the levels > 1
        obj.remove(Misc.group)  # remove the ball from the group
        obj.remove_sprite()
        utils.load_music(Snd.hurra).play()
        if (len(Misc.group) == (self.level > 1)):  # true if there are no balls in level 1 or there's in levels 2 and 3
            s = int(self.pointstoadd / self.throws)
            ## Consider this a placeholder, it doesn't work as it should
            #self.cpg.scorething(s)# Update the score display
            self.maxpoints += self.pointstoadd
            self.points += s
            self.tries += 1
            txt, sz = utils.text2surf(str(self.points) + '  ' + _("points"), 48, (8, 0, 120), bold=True)
            self.screen.blit(txt, (200, 200))
            pygame.display.update()
            pygame.time.wait(2000)
            # update the counter in the menubar
            Misc.execounter.increase_counter()
            if self.tries == self.n_balls:
                self.SPG.tellcore_level_end(
                    store_db=True)  #signal to childsplay that this level is ended. See return val of def loop
                if self.level == 3:
                    self.stop = 1
                    Misc.score = self.points
                    self.screen.blit(self.backgr, (0, 0))
                    MovingSign(_("You won!!"), 80, (100, 100), (100, 300), 1, (8, 0, 120), self.screen, self.backgr, 0)
                    txt, sz = utils.text2surf(str(self.points) + '  ' + _("points"), 48, (8, 0, 120), bold=True)
                    self.screen.blit(txt, (200, 200))
                    pygame.display.update()
                    for i in range(0, 3):
                        utils.load_music(Snd.hurra).play()
                        pygame.time.wait(1000)
            else:
                self.next_exercise()
